package javax.imageio.stream;

import sun.java2d.Disposer;
import com.sun.imageio.stream.StreamFinalizer;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;
import com.sun.imageio.stream.CloseableDisposerRecord;
import java.io.RandomAccessFile;

public class FileImageInputStream extends ImageInputStreamImpl
{
    private RandomAccessFile raf;
    private final Object disposerReferent;
    private final CloseableDisposerRecord disposerRecord;
    
    public FileImageInputStream(File file) throws FileNotFoundException, IOException {
        this((file == null) ? null : new RandomAccessFile(file, "r"));
    }
    
    public FileImageInputStream(RandomAccessFile randomAccessFile) {
        if (randomAccessFile == null) {
            throw new IllegalArgumentException("raf == null!");
        }
        this.raf = randomAccessFile;
        this.disposerRecord = new CloseableDisposerRecord(randomAccessFile);
        if (this.getClass() == FileImageInputStream.class) {
            Disposer.addRecord(this.disposerReferent = new Object(), this.disposerRecord);
        }
        else {
            this.disposerReferent = new StreamFinalizer(this);
        }
        return;
    }
    
    @Override
    public int read() throws IOException {
        this.checkClosed();
        this.bitOffset = 0;
        int read = this.raf.read();
        if (read != -1) {
            ++this.streamPos;
        }
        return read;
    }
    
    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        this.checkClosed();
        this.bitOffset = 0;
        int read = this.raf.read(b, off, len);
        if (read != -1) {
            this.streamPos += (long)read;
        }
        return read;
    }
    
    @Override
    public long length() {
        try {
            this.checkClosed();
            return this.raf.length();
        }
        catch (IOException ex) {
            return -1L;
        }
    }
    
    @Override
    public void seek(long pos) throws IOException {
        this.checkClosed();
        if (pos < this.flushedPos) {
            throw new IndexOutOfBoundsException("pos < flushedPos!");
        }
        this.bitOffset = 0;
        this.raf.seek(pos);
        this.streamPos = this.raf.getFilePointer();
        return;
    }
    
    @Override
    public void close() throws IOException {
        super.close();
        this.disposerRecord.dispose();
        this.raf = null;
    }
    
    @Override
    protected void finalize() throws Throwable {
    }
}
